var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var dumper_exports = {};
__export(dumper_exports, {
  Dumper: () => Dumper
});
module.exports = __toCommonJS(dumper_exports);
var import_archiver = __toESM(require("archiver"));
var import_dayjs = __toESM(require("dayjs"));
var import_fs = __toESM(require("fs"));
var import_promises = __toESM(require("fs/promises"));
var import_lodash = __toESM(require("lodash"));
var import_mkdirp = __toESM(require("mkdirp"));
var import_path = __toESM(require("path"));
var import_stream = __toESM(require("stream"));
var import_util = __toESM(require("util"));
var import_app_migrator = require("./app-migrator");
var import_collection_group_manager = require("./collection-group-manager");
var import_field_value_writer = require("./field-value-writer");
var import_utils = require("./utils");
const finished = import_util.default.promisify(import_stream.default.finished);
class Dumper extends import_app_migrator.AppMigrator {
  direction = "dump";
  async dumpableCollections() {
    const appCollectionGroups = import_collection_group_manager.CollectionGroupManager.getGroups(this.app);
    const { requiredGroups, optionalGroups } = import_collection_group_manager.CollectionGroupManager.classifyCollectionGroups(appCollectionGroups);
    const pluginsCollections = import_collection_group_manager.CollectionGroupManager.getGroupsCollections(appCollectionGroups);
    const userCollections = await this.getCustomCollections();
    return import_lodash.default.cloneDeep({
      requiredGroups,
      optionalGroups,
      userCollections: await Promise.all(
        userCollections.filter((collection) => !pluginsCollections.includes(collection)).map(async (name) => {
          const collectionInstance = await this.app.db.getRepository("collections").findOne({
            filterByTk: name
          });
          return {
            name,
            title: collectionInstance.get("title")
          };
        })
      )
    });
  }
  async dump(options) {
    const { requiredGroups, optionalGroups } = await this.dumpableCollections();
    const { selectedOptionalGroupNames, selectedUserCollections = [] } = options;
    const throughCollections = this.findThroughCollections(selectedUserCollections);
    const selectedOptionalGroups = optionalGroups.filter((group) => {
      return selectedOptionalGroupNames.some((selectedOptionalGroupName) => {
        const [namespace, functionKey] = selectedOptionalGroupName.split(".");
        return group.function === functionKey && group.namespace === namespace;
      });
    });
    const dumpedCollections = import_lodash.default.uniq(
      [
        import_collection_group_manager.CollectionGroupManager.getGroupsCollections(requiredGroups),
        import_collection_group_manager.CollectionGroupManager.getGroupsCollections(selectedOptionalGroups),
        selectedUserCollections,
        throughCollections
      ].flat()
    );
    for (const collection of dumpedCollections) {
      await this.dumpCollection({
        name: collection
      });
    }
    const mapGroupToMetaJson = (groups) => groups.map((group) => {
      const data = {
        ...group
      };
      if (group.delayRestore) {
        data["delayRestore"] = true;
      }
      return data;
    });
    await this.dumpMeta({
      requiredGroups: mapGroupToMetaJson(requiredGroups),
      selectedOptionalGroups: mapGroupToMetaJson(selectedOptionalGroups),
      selectedUserCollections
    });
    await this.dumpDb();
    const filePath = await this.packDumpedDir();
    await this.clearWorkDir();
    return filePath;
  }
  async dumpDb() {
    const db = this.app.db;
    const dialect = db.sequelize.getDialect();
    const sqlContent = [];
    if (dialect === "postgres") {
      const functions = await db.sequelize.query(
        `SELECT n.nspname                 AS function_schema,
                p.proname                 AS function_name,
                pg_get_functiondef(p.oid) AS def
         FROM pg_proc p
                LEFT JOIN pg_namespace n ON p.pronamespace = n.oid
         WHERE n.nspname NOT IN ('pg_catalog', 'information_schema')
         ORDER BY function_schema,
                  function_name;`,
        {
          type: "SELECT"
        }
      );
      for (const f of functions) {
        sqlContent.push(f["def"]);
      }
      const triggers = await db.sequelize.query(
        `select pg_get_triggerdef(oid)
         from pg_trigger`,
        {
          type: "SELECT"
        }
      );
      for (const t of triggers) {
        sqlContent.push(t["pg_get_triggerdef"]);
      }
      const views = await db.sequelize.query(
        `SELECT n.nspname    AS schema_name,
                v.viewname   AS view_name,
                v.definition AS view_definition
         FROM pg_views v
                JOIN
              pg_namespace n ON v.schemaname = n.nspname
         WHERE n.nspname NOT IN ('information_schema', 'pg_catalog')
         ORDER BY schema_name,
                  view_name;`,
        {
          type: "SELECT"
        }
      );
      for (const v of views) {
        sqlContent.push(`CREATE OR REPLACE VIEW ${v["view_name"]} AS ${v["view_definition"]}`);
      }
    }
    if (sqlContent.length > 0) {
      const dbDumpPath = import_path.default.resolve(this.workDir, "db.sql");
      await import_promises.default.writeFile(dbDumpPath, JSON.stringify(sqlContent), "utf8");
    }
  }
  async dumpMeta(additionalMeta = {}) {
    const metaPath = import_path.default.resolve(this.workDir, "meta");
    await import_promises.default.writeFile(
      metaPath,
      JSON.stringify({
        version: await this.app.version.get(),
        dialect: this.app.db.sequelize.getDialect(),
        ...additionalMeta
      }),
      "utf8"
    );
  }
  async dumpCollection(options) {
    const app = this.app;
    const dir = this.workDir;
    const collectionName = options.name;
    app.log.info(`dumping collection ${collectionName}`);
    const collection = app.db.getCollection(collectionName);
    if (!collection) {
      this.app.log.warn(`collection ${collectionName} not found`);
      return;
    }
    const columns = [...new Set(import_lodash.default.map(collection.model.tableAttributes, "field"))];
    if (columns.length == 0) {
      this.app.log.warn(`collection ${collectionName} has no columns`);
      return;
    }
    const collectionDataDir = import_path.default.resolve(dir, "collections", collectionName);
    await import_promises.default.mkdir(collectionDataDir, { recursive: true });
    const dataFilePath = import_path.default.resolve(collectionDataDir, "data");
    const dataStream = import_fs.default.createWriteStream(dataFilePath);
    const rows = await app.db.sequelize.query(
      (0, import_utils.sqlAdapter)(
        app.db,
        `SELECT *
         FROM ${collection.isParent() ? "ONLY" : ""} ${collection.quotedTableName()}`
      ),
      {
        type: "SELECT"
      }
    );
    for (const row of rows) {
      const rowData = JSON.stringify(
        columns.map((col) => {
          const val = row[col];
          const field = collection.getField(col);
          return field ? import_field_value_writer.FieldValueWriter.toDumpedValue(field, val) : val;
        })
      );
      dataStream.write(rowData + "\r\n", "utf8");
    }
    dataStream.end();
    await finished(dataStream);
    const meta = {
      name: collectionName,
      tableName: collection.model.tableName,
      count: rows.length,
      columns
    };
    await import_promises.default.writeFile(import_path.default.resolve(collectionDataDir, "meta"), JSON.stringify(meta), "utf8");
  }
  async packDumpedDir() {
    const dirname = import_path.default.resolve(process.cwd(), "storage", "duplicator");
    import_mkdirp.default.sync(dirname);
    const filePath = import_path.default.resolve(dirname, `dump-${(0, import_dayjs.default)().format("YYYYMMDDTHHmmss")}.${import_utils.DUMPED_EXTENSION}`);
    const output = import_fs.default.createWriteStream(filePath);
    const archive = (0, import_archiver.default)("zip", {
      zlib: { level: 9 }
    });
    output.on("close", function() {
      console.log("dumped file size: " + (0, import_utils.humanFileSize)(archive.pointer(), true));
    });
    output.on("end", function() {
      console.log("Data has been drained");
    });
    archive.on("warning", function(err) {
      if (err.code === "ENOENT") {
      } else {
        throw err;
      }
    });
    archive.on("error", function(err) {
      throw err;
    });
    archive.pipe(output);
    archive.directory(this.workDir, false);
    await archive.finalize();
    console.log("dumped to", filePath);
    return {
      filePath,
      dirname
    };
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Dumper
});
