var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var restorer_exports = {};
__export(restorer_exports, {
  Restorer: () => Restorer
});
module.exports = __toCommonJS(restorer_exports);
var import_decompress = __toESM(require("decompress"));
var import_fs = __toESM(require("fs"));
var import_promises = __toESM(require("fs/promises"));
var import_path = __toESM(require("path"));
var import_app_migrator = require("./app-migrator");
var import_collection_group_manager = require("./collection-group-manager");
var import_field_value_writer = require("./field-value-writer");
var import_utils = require("./utils");
class Restorer extends import_app_migrator.AppMigrator {
  direction = "restore";
  backUpFilePath;
  decompressed = false;
  importedCollections = [];
  constructor(app, options) {
    super(app, options);
    const { backUpFilePath } = options;
    if (backUpFilePath) {
      this.setBackUpFilePath(backUpFilePath);
    }
  }
  setBackUpFilePath(backUpFilePath) {
    if (import_path.default.isAbsolute(backUpFilePath)) {
      this.backUpFilePath = backUpFilePath;
    } else if (import_path.default.basename(backUpFilePath) === backUpFilePath) {
      const dirname = import_path.default.resolve(process.cwd(), "storage", "duplicator");
      this.backUpFilePath = import_path.default.resolve(dirname, backUpFilePath);
    } else {
      this.backUpFilePath = import_path.default.resolve(process.cwd(), backUpFilePath);
    }
  }
  async parseBackupFile() {
    await this.decompressBackup(this.backUpFilePath);
    return await this.getImportMeta();
  }
  async restore(options) {
    await this.decompressBackup(this.backUpFilePath);
    await this.importCollections(options);
    await this.importDb();
    await this.clearWorkDir();
  }
  async getImportPlugins() {
    const meta = await this.getImportCollectionMeta("applicationPlugins");
    const nameIndex = meta.columns.indexOf("name");
    const plugins = await this.getImportCollectionData("applicationPlugins");
    return ["core", ...plugins.map((plugin) => JSON.parse(plugin)[nameIndex])];
  }
  async getImportCustomCollections() {
    const collections = await this.getImportCollections();
    const meta = await this.getImportCollectionMeta("collections");
    const data = await this.getImportCollectionData("collections");
    return data.map((row) => JSON.parse(row)[meta.columns.indexOf("name")]).filter((name) => collections.includes(name));
  }
  async getImportCollectionTitle(collectionName) {
    const meta = await this.getImportCollectionMeta("collections");
    const data = await this.getImportCollectionData("collections");
    const index = meta.columns.indexOf("name");
    const row = data.find((row2) => JSON.parse(row2)[index] === collectionName);
    if (!row) {
      throw new Error(`Collection ${collectionName} not found`);
    }
    const titleIndex = meta.columns.indexOf("title");
    return JSON.parse(row)[titleIndex];
  }
  async getImportCollections() {
    const collectionsDir = import_path.default.resolve(this.workDir, "collections");
    return await import_promises.default.readdir(collectionsDir);
  }
  async getImportCollectionData(collectionName) {
    const dataFile = import_path.default.resolve(this.workDir, "collections", collectionName, "data");
    return await (0, import_utils.readLines)(dataFile);
  }
  async getImportCollectionMeta(collectionName) {
    const metaData = import_path.default.resolve(this.workDir, "collections", collectionName, "meta");
    return JSON.parse(await import_promises.default.readFile(metaData, "utf8"));
  }
  async getImportMeta() {
    const metaFile = import_path.default.resolve(this.workDir, "meta");
    return JSON.parse(await import_promises.default.readFile(metaFile, "utf8"));
  }
  async importCollections(options) {
    const importCollection = async (collectionName) => {
      const collectionMetaPath = import_path.default.resolve(this.workDir, "collections", collectionName, "meta");
      const metaContent = await import_promises.default.readFile(collectionMetaPath, "utf8");
      const meta = JSON.parse(metaContent);
      const tableName = this.app.db.utils.quoteTable(meta.tableName);
      try {
        if (this.app.db.inDialect("postgres")) {
          await this.app.db.sequelize.query(`ALTER TABLE IF EXISTS ${tableName} DISABLE TRIGGER ALL`);
        }
        await this.importCollection({
          name: collectionName
        });
      } catch (err) {
        this.app.log.warn(`import collection ${collectionName} failed`, {
          err
        });
      } finally {
        if (this.app.db.inDialect("postgres")) {
          await this.app.db.sequelize.query(`ALTER TABLE IF EXISTS ${tableName} ENABLE TRIGGER ALL`);
        }
      }
    };
    await importCollection("applicationPlugins");
    await this.app.reload();
    const { requiredGroups, selectedOptionalGroups } = await this.parseBackupFile();
    const delayGroups = [...requiredGroups, ...selectedOptionalGroups].filter((group) => group.delay);
    const delayCollections = import_collection_group_manager.CollectionGroupManager.getGroupsCollections(delayGroups);
    for (const collectionName of import_collection_group_manager.CollectionGroupManager.getGroupsCollections(requiredGroups).filter(
      (i) => !delayCollections.includes(i) && i != "applicationPlugins"
    )) {
      await importCollection(collectionName);
    }
    await this.app.db.getRepository("collections").load();
    await this.app.db.sync({
      force: false,
      alter: {
        drop: false
      }
    });
    const userCollections = options.selectedUserCollections || [];
    const throughCollections = this.findThroughCollections(userCollections);
    const customCollections = [
      ...import_collection_group_manager.CollectionGroupManager.getGroupsCollections(
        selectedOptionalGroups.filter((group) => {
          return options.selectedOptionalGroupNames.some((selectedOptionalGroupName) => {
            const [namespace, functionKey] = selectedOptionalGroupName.split(".");
            return group.function === functionKey && group.namespace === namespace;
          });
        })
      ),
      ...userCollections,
      ...throughCollections
    ];
    for (const collectionName of customCollections) {
      await importCollection(collectionName);
    }
    const appGroups = import_collection_group_manager.CollectionGroupManager.getGroups(this.app);
    for (const collectionGroup of delayGroups) {
      const appCollectionGroup = appGroups.find(
        (group) => group.namespace === collectionGroup.name && group.function === collectionGroup.function
      );
      await appCollectionGroup.delayRestore(this);
    }
    await this.emitAsync("restoreCollectionsFinished");
  }
  async decompressBackup(backupFilePath) {
    if (!this.decompressed)
      await (0, import_decompress.default)(backupFilePath, this.workDir);
  }
  async importCollection(options) {
    const app = this.app;
    const db = app.db;
    const collectionName = options.name;
    const dir = this.workDir;
    const collection = app.db.getCollection(collectionName);
    const collectionDataPath = import_path.default.resolve(dir, "collections", collectionName, "data");
    const collectionMetaPath = import_path.default.resolve(dir, "collections", collectionName, "meta");
    const metaContent = await import_promises.default.readFile(collectionMetaPath, "utf8");
    const meta = JSON.parse(metaContent);
    app.log.info(`collection meta ${metaContent}`);
    const addSchemaTableName = db.utils.addSchema(meta.tableName);
    const tableName = db.utils.quoteTable(meta.tableName);
    if (options.clear !== false) {
      let sql2 = `TRUNCATE TABLE ${tableName}`;
      if (app.db.inDialect("sqlite")) {
        sql2 = `DELETE
               FROM ${tableName}`;
      }
      await app.db.sequelize.query((0, import_utils.sqlAdapter)(app.db, sql2));
    }
    const rows = await (0, import_utils.readLines)(collectionDataPath);
    if (rows.length == 0) {
      app.logger.info(`${collectionName} has no data to import`);
      return;
    }
    const columns = meta["columns"];
    const fields = columns.map((column) => [column, collection.getField(column)]).reduce((carry, [column, type]) => {
      carry[column] = type;
      return carry;
    }, {});
    const rowsWithMeta = rows.map(
      (row) => JSON.parse(row).map((val, index) => [columns[index], val]).reduce((carry, [column, val]) => {
        const field = fields[column];
        carry[column] = field ? import_field_value_writer.FieldValueWriter.write(field, val) : val;
        return carry;
      }, {})
    ).filter((row) => {
      if (options.rowCondition) {
        return options.rowCondition(row);
      }
      return true;
    });
    if (rowsWithMeta.length == 0) {
      app.logger.info(`${collectionName} has no data to import`);
      return;
    }
    const model = collection.model;
    const fieldMappedAttributes = {};
    for (const attr in model.tableAttributes) {
      fieldMappedAttributes[model.rawAttributes[attr].field || attr] = model.rawAttributes[attr];
    }
    const sql = collection.model.queryInterface.queryGenerator.bulkInsertQuery(
      addSchemaTableName,
      rowsWithMeta,
      {},
      fieldMappedAttributes
    );
    if (options.insert === false) {
      return sql;
    }
    await app.db.sequelize.query(sql, {
      type: "INSERT"
    });
    const primaryKeyAttribute = collection.model.rawAttributes[collection.model.primaryKeyAttribute];
    if (primaryKeyAttribute && primaryKeyAttribute.autoIncrement) {
      this.on("restoreCollectionsFinished", async () => {
        if (this.app.db.inDialect("postgres")) {
          const sequenceNameResult = await app.db.sequelize.query(
            `SELECT column_default
             FROM information_schema.columns
             WHERE table_name = '${collection.model.tableName}'
               and "column_name" = 'id'
               and table_schema = '${app.db.options.schema || "public"}';`
          );
          if (sequenceNameResult[0].length) {
            const columnDefault = sequenceNameResult[0][0]["column_default"];
            if (columnDefault.includes(`${collection.model.tableName}_id_seq`)) {
              const regex = new RegExp(/nextval\('(.*)'::regclass\)/);
              const match = regex.exec(columnDefault);
              const sequenceName = match[1];
              const maxVal = await app.db.sequelize.query(
                `SELECT MAX("${primaryKeyAttribute.field}")
                 FROM ${tableName}`,
                {
                  type: "SELECT"
                }
              );
              const updateSeqSQL = `SELECT setval('${sequenceName}', ${maxVal[0]["max"]})`;
              await app.db.sequelize.query(updateSeqSQL);
            }
          }
        }
        if (this.app.db.inDialect("sqlite")) {
          await app.db.sequelize.query(
            `UPDATE sqlite_sequence
             set seq = (SELECT MAX("${primaryKeyAttribute.field}") FROM "${collection.model.tableName}")
             WHERE name = "${collection.model.tableName}"`
          );
        }
      });
    }
    app.logger.info(`${collectionName} imported with ${rowsWithMeta.length} rows`);
    this.importedCollections.push(collection.name);
  }
  async importDb() {
    const sqlFilePath = import_path.default.resolve(this.workDir, "db.sql");
    if (!import_fs.default.existsSync(sqlFilePath)) {
      return;
    }
    const queriesContent = await import_promises.default.readFile(sqlFilePath, "utf8");
    const queries = JSON.parse(queriesContent);
    for (const sql of queries) {
      try {
        this.app.log.info(`import sql: ${sql}`);
        await this.app.db.sequelize.query(sql);
      } catch (e) {
        if (e.name === "SequelizeDatabaseError") {
          this.app.logger.error(e.message);
        } else {
          throw e;
        }
      }
    }
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Restorer
});
